<?php

namespace App\Service;

use App\Exception;

class ConsumerService {
    
    private $branchFactory;
    private $consumerFactory;
    private $util;

    public function __construct (\App\Factory\BranchFactory $branchFactory,
                                 \App\Factory\ConsumerFactory $consumerFactory,
                                 \App\Util\Util $util) {
        $this->branchFactory   = $branchFactory;
        $this->consumerFactory = $consumerFactory;
        $this->util = $util;
    }

    public function processConsumer ($generalParams, $order)
    {
        $consumer          = $this->consumerFactory->getStandardConsumer($generalParams["CDFILIAL"]);
        $branchAddressInfo = $this->branchFactory->getAddressInfo($generalParams["CDFILIAL"]);
        $address           = $this->processAddressInformation($order, $branchAddressInfo, $generalParams);

        if (empty($consumer["CDMUNICIPIO"]))
            $consumer["CDMUNICIPIO"] = $this->consumerFactory->getCityCode($order["deliveryAddress"]["state"], $order["deliveryAddress"]["city"]);

        if (empty($consumer["CDMUNICIPIO"]))
            $consumer["CDMUNICIPIO"] = $this->branchFactory->getBranchCityCode($generalParams["CDFILIAL"]);
        
        $consumer["DSAREAATEND"] = "";
        
        if($order["type"] == "DELIVERY") {
            $areaAtend = $this->getDeliveryAreas($order["deliveryAddress"]["coordinates"]);
        
            if((isset($areaAtend["INSIDE"])) && ($areaAtend["INSIDE"])) {
                $consumer["DSAREAATEND"] = $areaAtend["NAMEAREA"];
            }
        }

        return array_merge($address, $consumer);
    }
    
    private function processAddressInformation ($order, $branchAddressInfo, $generalParams)
    {
        $consumer = null;
        
        $consumer["street"]          = $order["deliveryAddress"]["streetName"];
        $consumer["number"]          = $order["deliveryAddress"]["streetNumber"];
        $consumer["neighborhood"]    = $order["deliveryAddress"]["neighborhood"];
        $consumer["city"]            = $order["deliveryAddress"]["city"];
        $consumer["cep"]             = $order["deliveryAddress"]["postalCode"];
        $consumer["complement"]      = $order["deliveryAddress"]["complement"];
        $consumer["reference_point"] = $order["deliveryAddress"]["reference"];
        
        $consumer["CDMUNICIPIO"]     = $branchAddressInfo["CDMUNICIPIO"];
        $consumer["SGESTADO"]        = $branchAddressInfo["SGESTADO"];
        $consumer["CDBAIRRO"]        = $this->consumerFactory->getNeighborhoodCode($consumer);

        
        if ($order["type"] == "TAKE") {
            $consumer["cep"]          = preg_replace("/[^0-9]/", "", $branchAddressInfo["NRCEPFILI"]);
            $consumer["street"]       = $branchAddressInfo["DSENDEFILI"];
            $consumer["neighborhood"] = $branchAddressInfo["NMBAIRFILI"];
        }
        
        return $consumer;
    }
    
    private function getDeliveryAreas($locationCoords)
    {
        $areas = $this->branchFactory->getDeliveryAreas();

        if(!$areas['error']) {
            $inside = [];

            $areas = $areas['areas'];
            foreach($areas as $cordinates) {
                $stringCoordinates   = $cordinates["COORDAREA"];
                $stringCoordinates   = str_replace("['","",$stringCoordinates);
                $stringCoordinates   = str_replace("']","",$stringCoordinates);
                $stringCoordinates   = str_replace("','",'$',$stringCoordinates);
                $polyCoordinates     = [];
                $explodedCoordinates = explode('$',$stringCoordinates);  
                
                foreach($explodedCoordinates as $latLongStr) {
                    $latLong = explode(',',$latLongStr);
                    array_push($polyCoordinates, $latLong);
                }

                $inside = $this->inside($locationCoords, $polyCoordinates);
               
                if($inside) {
                    $inside = [
                        "INSIDE"    => $inside,
                        "NAMEAREA"  => $cordinates["DSAREAATEND"],
                    ];

                    return $inside;    
                }
            }

            return $inside;
        } else {
            throw new \Exception($areas["message"], 992); 
        }
    }
    
    private function inside($point, $vs)
    {
        $x = (float)$point["latitude"];
        $y = (float)$point["longitude"];

        $inside = false;
        for ($i = 0, $j = count($vs) - 1; $i < count($vs); $j = $i++) {
            $xi = (float)$vs[$i][0];
            $yi = (float)$vs[$i][1];
            $xj = (float)$vs[$j][0];
            $yj = (float)$vs[$j][1];

            $intersect = (($yi > $y) != ($yj > $y)) && ($x < ($xj - $xi) * ($y - $yi) / ($yj - $yi) + $xi);
            if ($intersect) 
                $inside = !$inside;
        }
        
        return $inside;
    }
}
